//************************************************************************
// KS108.H
// LowLevel Routinen fr LCD-Displays mit KS0108
//
//
// hk@holger-klabunde.de
// http://www.holger-klabunde.de
// 13.12.2006
// Compiler WinAVR 3.4.5
//************************************************************************

#ifndef KS108_H
#define KS108_H

//#####################################################
//# Number of NOP's below is based on 11.059MHz Clock #
//#####################################################

// Only define ONE of these !!
#define LCD_WIDTH_240         //Display Width for 240x64
//#define LCD_WIDTH_128         //Display Width for 128x64

// KS108 displays have one or more ks108 controllers.
// 240x64 has 4 controllers, 128x64 has 2 controllers
#ifdef LCD_WIDTH_240
 #define LCD_WIDTH		240
 #define MAX_CONTROLLERS	4
#endif

#ifdef LCD_WIDTH_128
 #define LCD_WIDTH		128
 #define MAX_CONTROLLERS	2
#endif

#define LCD_HEIGHT	64   //Display Hhe

// security check !
#if defined (LCD_WIDTH_240) && defined (LCD_WIDTH_128)
 #error "Define LCD_WIDTH_240 or LCD_WIDTH_128 only in ks108.h, NOT both !"
#endif


#if defined (__AVR_ATmega128__) || defined (__AVR_ATmega64__)

#define INIT_CONTROL_PINS() { DDRD=0xFF; DDRF=0xFF; RSTB_ON(); } //control pins to output

#define DATA_PORT	PORTD
#define DATA_DIR_IN()  	{ DDRD=0x00; DATA_PORT=0xFF; } // set io-pins to inputs with pullups
#define DATA_DIR_OUT()	DDRD=0xFF; // set io-pins to outputs
#define READ_DATA()   	PIND	  // read PIN, ! NOT ! PORT
#define WRITE_DATA(a)  	DATA_PORT=(a); // write to data port

// Zu meiner Erinnerung:
// Im Assemblerlisting wurde PORTF gelesen, modifiziert, und zurckgeschrieben.
// Wieso kein krzeres "sbi PORTF,BIT" ??? Datenblatt lesen !
// sbi/cbi funktionieren nur mit SFR's von 0x00 bis 0x1F. PORTF liegt darber.
// Es bringt jetzt aber hier auch nichts PORTF und PORTD zu tauschen (habs versucht).
// Das was man dann gewinnt mu man in Wait_for_E() wieder draufgeben weil das Timing
// fr "E_ON/E_OFF" nicht mehr stimmt.
//
// Fr andere Projekte sollte man aber in Erinnerung behalten das PORTF nicht fr
// extrem schnelle Anwendungen geeignet ist ;)

#define RS_BIT		7
#define RS_PORT		PORTF
#define RS_ON()	       	{ sbi(RS_PORT,RS_BIT); }
#define RS_OFF()       	{ cbi(RS_PORT,RS_BIT); }

#define RW_BIT		6
#define RW_PORT		PORTF
#define RW_ON()	       	{ sbi(RW_PORT,RW_BIT); NOP(); NOP(); }
#define RW_OFF()       	{ cbi(RW_PORT,RW_BIT); NOP(); NOP(); } //min 140ns

#define E_BIT		5
#define E_PORT		PORTF
#define E_ON()	       	{ sbi(E_PORT,E_BIT); Wait_for_E(); } //min 450ns
#define E_OFF()       	{ cbi(E_PORT,E_BIT); Wait_for_E(); } //min 450ns

// Fr Displays ohne RSTB (Reset) Pin folgende vier Zeilen einfach
// wegkommentieren.
#define RSTB_BIT	4
#define RSTB_PORT	PORTF
#define RSTB_ON()      	sbi(RSTB_PORT,RSTB_BIT);
#define RSTB_OFF()     	cbi(RSTB_PORT,RSTB_BIT);

#define CS1_BIT		0
#define CS1_PORT	PORTF
#define CS1_ON()      	sbi(CS1_PORT,CS1_BIT);
#define CS1_OFF()     	cbi(CS1_PORT,CS1_BIT);

#define CS2_BIT		1
#define CS2_PORT	PORTF
#define CS2_ON()      	sbi(CS2_PORT,CS2_BIT);
#define CS2_OFF()     	cbi(CS2_PORT,CS2_BIT);

#if defined (LCD_WIDTH_240)
 #define CS3_BIT	2
 #define CS3_PORT	PORTF
 #define CS3_ON()      	sbi(CS3_PORT,CS3_BIT);
 #define CS3_OFF()     	cbi(CS3_PORT,CS3_BIT);

 #define CS4_BIT	3
 #define CS4_PORT	PORTF
 #define CS4_ON()      	sbi(CS4_PORT,CS4_BIT);
 #define CS4_OFF()     	cbi(CS4_PORT,CS4_BIT);
#endif

#elif defined (__AVR_ATmega32__)
//todo 
#  error "No pin definitions for KS108 display and ATMega32 in ks108.h"

#else
#  error "processor type not defined in ks108.h"
#endif

extern unsigned char ReadDisplay(void);
extern void WriteDisplay(unsigned char dat);
extern unsigned char SetPixel(unsigned char xpos, unsigned char ypos, unsigned char mode);
extern unsigned char GetPixel(unsigned char xpos, unsigned char ypos);
extern unsigned char GetChipNumber(unsigned char xpos);
extern void SetChipNumber(unsigned char num);
extern unsigned char GetPageNumber(unsigned char ypos);
extern unsigned char SetPageNumber(unsigned char num);
extern unsigned char SetRow(unsigned char xpos);
extern unsigned char SetPosition(unsigned char xpos, unsigned char ypos);
extern void DisplayOn(void);
extern void ClearScreen(void);
extern unsigned char CheckBusy(void);

#endif
